<?php
/**
 * DTWPB_Current_Query_Renderer
 *
 * @package WooCommerce-Builder
 *
 */

defined( 'ABSPATH' ) || exit;

class DTWPB_Current_Query_Renderer extends \WC_Shortcode_Products {
	
	private $settings = [];
	
	public function __construct( $settings = array(), $type = 'products' ){
		$this->type = $type;
		$this->settings = $settings;
		$this->attributes = $this->parse_attributes( [
			'columns' => $settings['columns'],
			'rows' => $settings['rows'],
			'paginate' => $settings['paginate'],
			'class' => $settings['class'],
			'cache' => false,
		] );
		$this->query_args = $this->parse_query_args();
		
	}
	
	/**
	 * Override the original `get_query_results`
	 * with modifications that:
	 * 1. Remove `pre_get_posts` action if `is_added_product_filter`.
	 *
	 * @return bool|mixed|object
	 */
	
	protected function get_query_results() {
		
		$results = parent::get_query_results();
		// Start edit.
		if ( $this->is_added_product_filter ) {
			remove_action( 'pre_get_posts', [ wc()->query, 'product_query' ] );
		}
		// End edit.
	
		return $results;
	}
	
	protected function parse_query_args() {
		$settings = $this->settings;
		$query_args = [
			'post_type' => 'product',
			'post_status' => 'publish',
			'ignore_sticky_posts' => true,
			'no_found_rows' => false === wc_string_to_bool( $this->attributes['paginate'] ),
		];
		
		if ( ! is_page( wc_get_page_id( 'shop' ) ) ) {
			$query_args = $GLOBALS['wp_query']->query_vars;
		}

		// Fix for parent::get_transient_name.
		if ( ! isset( $query_args['orderby'] ) ) {
			$query_args['orderby'] = '';
			$query_args['order'] = '';
		}

		add_action( 'pre_get_posts', [ wc()->query, 'product_query' ] );
		$this->is_added_product_filter = true;
			
		
		if ( 'yes' === $settings['paginate'] ) {
			$page = absint( empty( $_GET['product-page'] ) ? 1 : $_GET['product-page'] );
	 
			if ( 1 < $page ) {
				$query_args['paged'] = $page;
			}
	
			if ( 'yes' !== $settings['allow_order'] ) {
				remove_action( 'woocommerce_before_shop_loop', 'woocommerce_catalog_ordering', 30 );
			}
	
			if ( 'yes' !== $settings['show_result_count'] ) {
				remove_action( 'woocommerce_before_shop_loop', 'woocommerce_result_count', 20 );
			}
		}
		$query_args['posts_per_page'] = intval( $settings['columns'] * $settings['rows'] );
		
		// Always query only IDs.
		$query_args['fields'] = 'ids';
		return $query_args;
	}
	
	
	
	/**
	 * Loop over found products.
	 *
	 * @since  3.2.0
	 * @return string
	 */
	protected function product_loop() {
		$columns  = absint( $this->attributes['columns'] );
		$classes  = $this->get_wrapper_classes( $columns );
		$products = $this->get_query_results();
		
		$isTheme = wc_get_theme_slug_for_templates();
		ob_start();
	
		if ( $products && $products->ids ) {
			// Prime caches to reduce future queries.
			if ( is_callable( '_prime_post_caches' ) ) {
				_prime_post_caches( $products->ids );
			}
	
			// Setup the loop.
			wc_setup_loop(
			array(
			'columns'      => $columns,
			'name'         => $this->type,
			'is_shortcode' => true,
			'is_search'    => false,
			'is_paginated' => wc_string_to_bool( $this->attributes['paginate'] ),
			'total'        => $products->total,
			'total_pages'  => $products->total_pages,
			'per_page'     => $products->per_page,
			'current_page' => $products->current_page,
			)
			);
	
			$original_post = $GLOBALS['post'];
	
			do_action( "woocommerce_shortcode_before_{$this->type}_loop", $this->attributes );
	
			// Fire standard shop loop hooks when paginating results so we can show result counts and so on.
			if ( wc_string_to_bool( $this->attributes['paginate'] ) ) {
				
				if ( wc_string_to_bool( $this->settings['allow_order'] ) ) {
					woocommerce_catalog_ordering();
				}
				if ( wc_string_to_bool( $this->settings['show_result_count'] ) ) {
					woocommerce_result_count();
				}
			}
	
			woocommerce_product_loop_start();
	
			if ( wc_get_loop_prop( 'total' ) ) {
				foreach ( $products->ids as $product_id ) {
					$GLOBALS['post'] = get_post( $product_id ); // WPCS: override ok.
					setup_postdata( $GLOBALS['post'] );
	
					// Set custom product visibility when quering hidden products.
					add_action( 'woocommerce_product_is_visible', array( $this, 'set_product_as_visible' ) );
	
					// Render product template.
					wc_get_template_part( 'content', 'product' );
	
					// Restore product visibility.
					remove_action( 'woocommerce_product_is_visible', array( $this, 'set_product_as_visible' ) );
				}
			}
	
			$GLOBALS['post'] = $original_post; // WPCS: override ok.
			woocommerce_product_loop_end();
	
			// Fire standard shop loop hooks when paginating results so we can show result counts and so on.
			if ( wc_string_to_bool( $this->attributes['paginate'] ) ) {
				if( $this->settings['query_post_type'] == 'preview' ){
					woocommerce_pagination();
				}else{
					if( $isTheme == 'big-event' ){
						big_event_numeric_pagination();
					}else{
						do_action( 'woocommerce_after_shop_loop' );
					}
				}
			}
	
			do_action( "woocommerce_shortcode_after_{$this->type}_loop", $this->attributes );
	
			wp_reset_postdata();
			wc_reset_loop();
		} else {
			if ( $this->settings['nothing_found_message'] ) {
				echo '<div class="woocommerce-info elementor-nothing-found elementor-products-nothing-found">' . esc_html( $this->settings['nothing_found_message'] ) . '</div>';
			}else{
				do_action( "woocommerce_shortcode_{$this->type}_loop_no_results", $this->attributes );
			}
		}
	
		return '<div class="' . esc_attr( implode( ' ', $classes ) ) . '">' . ob_get_clean() . '</div>';
	}
	
}